@@ -180,10 +180,6 @@ class ElementBase(object):
             dependencies.extend(c.dependencies(memo))
         return dependencies
 
-    def compile(self, compiler):
-        raise NotImplementedError("Call to virtual method compile()"
-                                  " in base class ElementBase")
-
     def gstring(self):
         """
             Returns a formatted grammar string of the contents
@@ -281,14 +277,15 @@ class Sequence(ElementBase):
         the recognition in the order that they were given in the
         *children* constructor argument.
 
-        Example usage:
-        >>> from dragonfly.test import ElementTester
-        >>> seq = Sequence([Literal("hello"), Literal("world")])
-        >>> test_seq = ElementTester(seq)
-        >>> test_seq.recognize("hello world")
-        ['hello', 'world']
-        >>> test_seq.recognize("hello universe")
-        RecognitionFailure
+        Example usage::
+
+           >>> from dragonfly.test import ElementTester
+           >>> seq = Sequence([Literal("hello"), Literal("world")])
+           >>> test_seq = ElementTester(seq)
+           >>> test_seq.recognize("hello world")
+           ['hello', 'world']
+           >>> test_seq.recognize("hello universe")
+           RecognitionFailure
 
     """
 
@@ -604,7 +601,7 @@ class Repetition(Sequence):
                             " ElementBase instance." % self)
         assert isinstance(min, integer_types)
         assert max is None or isinstance(max, integer_types)
-        assert max is None or min < max
+        assert max is None or min < max, "min must be less than max"
 
         self._child = child
         self._min = min
@@ -615,7 +612,7 @@ class Repetition(Sequence):
         optional_length = self._max - self._min - 1
         if optional_length > 0:
             element = Optional(child)
-            for index in range(optional_length):
+            for index in range(optional_length-1):
                 element = Optional(Sequence([child, element]))
 
             if self._min >= 1:
@@ -718,6 +715,10 @@ class Literal(ElementBase):
         Element class representing one or more literal words which must be
         said exactly by the speaker as given.
 
+        Quoted words can be used to potentially improve accuracy. This
+        currently only has an effect if using the Natlink SR engine
+        back-end.
+
         Constructor arguments:
          - *text* (*str*) --
            the words to be said by the speaker
@@ -728,17 +729,70 @@ class Literal(ElementBase):
          - *default* (*object*, default: *None*) --
            the default value used if this element is optional and wasn't
            spoken
+         - *quote_start_str* (*str*, default: \") --
+           the string used for specifying the start of quoted words.
+         - *quote_end_str* (*str*, default: \") --
+           the string used for specifying the end of quoted words.
+         - *strip_quote_strs* (*bool*, default: *True*) --
+           whether the start and end quote strings should be stripped from
+           this element's word lists.
 
     """
 
-    def __init__(self, text, name=None, value=None, default=None):
+    def __init__(self, text, name=None, value=None, default=None,
+                 quote_start_str='"', quote_end_str='"',
+                 strip_quote_strs=True):
         ElementBase.__init__(self, name, default=default)
         self._value = value
 
         if not isinstance(text, string_types):
             raise TypeError("Text of %s object must be a"
                             " string." % self)
-        self._words = text.split()
+
+        # Construct the words and extended words lists. The latter includes
+        # quoted words as single items.
+        words = []
+        words_ext = []
+        current_quoted_sequence = []
+        for word in text.split():
+            # Begin quoted words.
+            if word.startswith(quote_start_str):
+                current_quoted_sequence.append(word)
+
+            # End quoted words.
+            elif current_quoted_sequence and word.endswith(quote_end_str):
+                current_quoted_sequence.append(word)
+                quoted_words = " ".join(current_quoted_sequence)
+
+                # Strip quote start and end strings if specified.
+                if strip_quote_strs:
+                    quoted_words = quoted_words[len(quote_start_str):
+                                                len(quote_end_str) * -1]
+
+                # Add the words to both lists.
+                words.extend(quoted_words.split())
+                words_ext.append(quoted_words)
+
+                # Clear current sequence list.
+                del current_quoted_sequence[:]
+
+            # Continuing quoted words.
+            elif current_quoted_sequence:
+                current_quoted_sequence.append(word)
+
+            # Unquoted words.
+            else:
+                words.append(word)
+                words_ext.append(word)
+
+        # Handle unfinished quoted words sequence by treating it as normal.
+        if current_quoted_sequence:
+            words.extend(current_quoted_sequence)
+            words_ext.extend(current_quoted_sequence)
+
+        # Set both lists.
+        self._words = words
+        self._words_ext = words_ext
 
     #-----------------------------------------------------------------------
     # Methods for runtime introspection.
@@ -746,7 +800,16 @@ class Literal(ElementBase):
     def __repr__(self):
         return "%s(%r)" % (self.__class__.__name__, self.words)
 
-    words = property(lambda self: self._words)
+    words = property(
+        lambda self: self._words,
+        doc="The words to be said by the speaker."
+    )
+
+    words_ext = property(
+        lambda self: self._words_ext,
+        doc="The words to be said by the speaker, with any quoted words as "
+        "single items. This is extends the :py:attr:`~words` property."
+    )
 
     #-----------------------------------------------------------------------
     # Methods for load-time setup.
@@ -765,7 +828,11 @@ class Literal(ElementBase):
 
         # Iterate through this element's words.
         # If all match, success.  Else, failure.
-        for i in range(len(self._words)):
+        if state.engine and state.engine.quoted_words_support:
+            words = self._words_ext
+        else:
+            words = self._words
+        for i in range(len(words)):
             word = state.word(i)
 
             # If word isn't None, make it lowercase.
@@ -773,12 +840,12 @@ class Literal(ElementBase):
                 word = word.lower()
 
             # If the words are not the same, then this is a decode failure.
-            if word != self._words[i].lower():
+            if word != words[i].lower():
                 state.decode_failure(self)
                 return
 
         # All words matched, success.
-        state.next(len(self._words))
+        state.next(len(words))
         state.decode_success(self)
         yield state
         state.decode_retry(self)
@@ -915,9 +982,6 @@ class ListRef(ElementBase):
         memo.add(self._id)
         return [self._list]
 
-    def compile(self, compiler):
-        compiler.add_list(self._list.name)
-
     def gstring(self):
         return "{" + self._list.name + "}"
 
@@ -1006,9 +1070,6 @@ class Empty(ElementBase):
     #-----------------------------------------------------------------------
     # Methods for load-time setup.
 
-    def compile(self, compiler):
-        pass
-
     def gstring(self):
         return "<Empty()>"
 
